# -*- coding: utf-8 -*-
"""
Created on Mon Jun 26 13:40:42 2017

@author: Pierre Héroux and Maroua Mehri
"""
from __future__ import division
from jinja2 import Environment, FileSystemLoader

def hba_evaluate(result_directory,gt_directory, participant_id, book_id):
    """
    Evaluate the results provided by the files in result_directory with respect 
    to the ground truth information given by the files in gt_directory.
    """
    
    import os
    import os.path
    import pandas as pd
    import numpy as np
    
    total = np.zeros((2,3),dtype=np.int)
        
    # Check whether result_directory and gt_directory are directories
    if not os.path.isdir(result_directory):
        print("The result folder is not a directory")
        return
    
    if not os.path.isdir(gt_directory):
        print("The gt folder is not a directory")
        return
    
    # For each file of the ground truth directory
    for f in sorted(os.listdir(gt_directory)): 
        # Read the corresponding file in the ground truth directory 
        gt_data = pd.read_table(gt_directory+'/'+f,names=['x','y','true_class'])
        gt_data['true_class'] = gt_data['true_class'].astype(int)
        gt_data.loc[gt_data['true_class'] != 1, 'true_class'] = 2
        gt_data=gt_data.set_index(['x','y'])
        
        # Read the corresponding file in the result directory if any
        if os.path.isfile(result_directory+'/'+f):
            result_data = pd.read_table(result_directory+'/'+f,names=['x','y','predicted_class'])
        else:
            result_data = pd.DataFrame(columns=['x','y','predicted_class'])

        # Remove multiple entries for (x,y)
        result_data['predicted_class'] = result_data['predicted_class'].astype(int)
        result_data.loc[result_data['predicted_class'] > 2, 'predicted_class'] = 3
        result_data = result_data.set_index(['x','y'])
        result_data = result_data[~result_data.index.duplicated(keep=False)]
        
        gt_data = gt_data.reset_index()
        result_data = result_data.reset_index()
        
        # Compare result with ground truth 
        merge = pd.merge(gt_data, result_data,on=['x','y'],how='left')
        merge = merge.replace(np.nan,3,regex=True)        
        
        if len(merge) > 0:
            count = merge.groupby(['true_class','predicted_class']).size()
        
            for k in count.keys():
                total[int(k[0])-1,int(k[1])-1] = total[int(k[0])-1,int(k[1])-1]+count.get(k)
                
    print(total)

    # Total is the confusion matrix    
    
    sum_lines = np.sum(total,axis=1)
    sum_columns = np.sum(total,axis=0)

    result = {}    
    TP = 0
    macro_precision = 0
    macro_recall = 0
    micro_precision = 0
    micro_recall = 0    
    
    for i in range(2) :
        true_effective = sum_lines[i]
        estimated_effective = sum_columns[i]
        true_positive = total[i][i]
        precision = true_positive / estimated_effective if estimated_effective != 0 else np.nan
        recall = true_positive / true_effective if true_effective != 0 else np.nan
        F1 = 2*precision*recall / (precision+recall) if precision+recall != 0 else np.nan

        result['precision_'+str(i+1)] = precision
        result['recall_'+str(i+1)] = recall
        result['F1_'+str(i+1)] = F1
        if np.isnan(precision) == False:
            macro_precision = macro_precision + precision
        if np.isnan(recall) == False:
            macro_recall = macro_recall + recall
        if np.isnan(precision * estimated_effective) == False:
            micro_precision = micro_precision + precision * estimated_effective
        if np.isnan(recall * true_effective) == False:
            micro_recall = micro_recall + recall * true_effective
        TP = TP+total[i][i]

    mean_precision = macro_precision / 2
    mean_recall = macro_recall / 2
    weighted_precision = micro_precision / np.sum(sum_columns[:-1]) if np.sum(sum_columns[:-1]) != 0 else 0
    weighted_recall = micro_recall / np.sum(sum_lines) if np.sum(sum_lines) != 0 else 0
    mean_accuracy = TP / np.sum(sum_lines) if np.sum(sum_lines) != 0 else 0
    average_fmeasure = 2*mean_precision*mean_recall / (mean_precision+mean_recall) if mean_precision != 0 and mean_recall != 0 else 0
    weighted_fmeasure = 2*weighted_precision*weighted_recall / (weighted_precision+weighted_recall) if weighted_precision != 0 and weighted_recall != 0 else 0
        
    result['Mean_accuracy']= mean_accuracy
    result['Average_precision'] = mean_precision
    result['Average_recall'] = mean_recall
    result['Average_FMeasure'] = average_fmeasure
    result['Weighted_precision'] = weighted_precision
    result['Weighted_recall'] = weighted_recall
    result['Weighted_FMeasure'] = weighted_fmeasure
    result['Participant_number'] = participant_id
    result['Book_number'] = book_id    
    
    return result

env = Environment(loader=FileSystemLoader('.'))
template = env.get_template("hba_evaluation_report_template_challenge1.html")

res = hba_evaluate(result_directory = './result',gt_directory = './gt', participant_id = 1, book_id = 1)

template_vars = {'Participant_number': res['Participant_number'], 
                 'book_number': res['Book_number'],
                 'class1_prec':res['precision_1'],
                 'class1_recall':res['recall_1'],
                 'class1_f1':res['F1_1'],
                 'class2_prec':res['precision_2'],
                 'class2_recall':res['recall_2'],
                 'class2_f1':res['F1_2'],
                 'Mean_accuracy':res['Mean_accuracy'],
                 'Weighted_precision':res['Weighted_precision'],
                 'Average_precision':res['Average_precision'],
                 'Weighted_recall':res['Weighted_recall'],
                 'Weighted_FMeasure':res['Weighted_FMeasure'],
                 'Average_FMeasure':res['Average_FMeasure'],
                 'Average_recall':res['Average_recall']
}
                 
html_out = template.render(template_vars)
output_file_name = 'hba_evaluation_report_challenge1_participant_'+str(res['Participant_number'])+'_book_'+str(res['Book_number'])+'.html'
fd = open(output_file_name,'w')
fd.write(html_out)
fd.close()
